
import UIKit
import CoreData

class BooksViewController: UITableViewController {
   var context: NSManagedObjectContext!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext
      
      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "booksCell")
      prepareDataSource()
   }
   override func viewWillAppear(_ animated: Bool) {
      super.viewWillAppear(animated)
      Task(priority: .high) {
         await loadRequest()

         await MainActor.run {
            prepareSnapshot()
         }
      }
   }
   func prepareDataSource() {
      AppData.dataSourceBooks = UITableViewDiffableDataSource<Sections, NSManagedObjectID>(tableView: tableView) { tableView, indexPath, itemID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "booksCell", for: indexPath)
         if let item = AppData.listOfBooks.first(where: { $0.objectID == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.title
            config.secondaryText = item.author?.name ?? "Undefined"
            config.secondaryTextProperties.color = .systemGray

            if let data = item.thumbnail, let image = UIImage(data: data) {
               config.image = image
            } else {
               config.image = UIImage(named: "nothumbnail")
            }
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func loadRequest() async {
      let search = "stephen"

      await context.perform {
         let request: NSFetchRequest<Books> = Books.fetchRequest()
         request.predicate = NSPredicate(format: "author.name BEGINSWITH[c] %@", search)
         do {
            AppData.listOfBooks = try self.context.fetch(request)
         } catch {
            print("Error: \(error)")
         }
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>()
      snapshot.appendSections([.main])
      snapshot.appendItems(AppData.listOfBooks.map({ $0.objectID }))
      AppData.dataSourceBooks.apply(snapshot)
   }
}

