
import UIKit
import CoreData

class BooksViewController: UITableViewController {
   var context: NSManagedObjectContext!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "booksCell")
      prepareDataSource()
   }
   override func viewWillAppear(_ animated: Bool) {
      super.viewWillAppear(animated)
      Task(priority: .high) {
         await AppData.loadRequestBooks(context: context)

         await MainActor.run {
            AppData.prepareSnapshotBooks()
         }
      }
   }
   func prepareDataSource() {
      AppData.dataSourceBooks = MyDataSource(tableView: tableView) { tableView, indexPath, itemID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "booksCell", for: indexPath)
         if let item = AppData.listOfBooks.first(where: { $0.objectID == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.title
            config.secondaryText = item.author?.name ?? "Undefined"
            config.secondaryTextProperties.color = .systemGray

            if let data = item.thumbnail, let image = UIImage(data: data) {
               config.image = image
            } else {
               config.image = UIImage(named: "nothumbnail")
            }
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      performSegue(withIdentifier: "showEditBook", sender: self)
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "showEditBook" {
         if let path = tableView.indexPathForSelectedRow {
            if let itemID = AppData.dataSourceBooks.itemIdentifier(for: path) {
               if let item = AppData.listOfBooks.first(where: { $0.objectID == itemID }) {
                  let controller = segue.destination as! EditBookViewController
                  controller.selectedBook = item
               }
            }
         }
      }
   }
   @IBAction func editBooks(_ sender: UIBarButtonItem) {
      let editing = !tableView.isEditing
      tableView.setEditing(editing, animated: true)
   }
}

