
import UIKit
import CoreData

class BooksViewController: UITableViewController, NSFetchedResultsControllerDelegate, UISearchResultsUpdating, UISearchBarDelegate {
   var context: NSManagedObjectContext!
   var searchController: UISearchController!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext
      searchController = UISearchController(searchResultsController: nil)
      searchController.searchResultsUpdater = self
      searchController.obscuresBackgroundDuringPresentation = false
      searchController.searchBar.delegate = self
      navigationItem.searchController = searchController

      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "booksCell")
      prepareDataSource()
      prepareFetchedController()
   }
   override func viewWillAppear(_ animated: Bool) {
      super.viewWillAppear(animated)
      do {
         try AppData.fetchedController.performFetch()
      } catch {
         print("Error")
      }
   }
   func prepareDataSource() {
      AppData.dataSourceBooks = MyDataSource(tableView: tableView) { tableView, indexPath, itemID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "booksCell", for: indexPath)
         if let item = try? self.context.existingObject(with: itemID) as? Books {
            var config = cell.defaultContentConfiguration()
            config.text = item.title
            config.secondaryText = item.author?.name ?? "Undefined"
            config.secondaryTextProperties.color = .systemGray

            if let data = item.thumbnail, let image = UIImage(data:data){
               config.image = image
            } else {
               config.image = UIImage(named: "nothumbnail")
            }
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareFetchedController() {
      let request: NSFetchRequest<Books> = Books.fetchRequest()
      let sort1 = NSSortDescriptor(key: "author.name", ascending: true, selector: #selector(NSString.caseInsensitiveCompare(_:)))
      let sort2 = NSSortDescriptor(key: "title", ascending: true, selector: #selector(NSString.caseInsensitiveCompare(_:)))
      request.sortDescriptors = [sort1, sort2]

      AppData.fetchedController = NSFetchedResultsController(fetchRequest: request, managedObjectContext: context, sectionNameKeyPath: "author.name", cacheName: nil)
      AppData.fetchedController.delegate = self
   }
   func controller(_ controller: NSFetchedResultsController<NSFetchRequestResult>, didChangeContentWith snapshot: NSDiffableDataSourceSnapshotReference) {
      let newsnapshot = snapshot as NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>
      AppData.dataSourceBooks.apply(newsnapshot, animatingDifferences: true)
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      performSegue(withIdentifier: "showEditBook", sender: self)
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "showEditBook" {
         if let path = tableView.indexPathForSelectedRow {
            if let itemID = AppData.dataSourceBooks.itemIdentifier(for: path) {
               if let item = try? context.existingObject(with: itemID) as? Books {
                  let controller = segue.destination as! EditBookViewController
                  controller.selectedBook = item
               }
            }
         }
      }
   }
   func updateSearchResults(for searchController: UISearchController) {
      if let text = searchController.searchBar.text {
         let search = text.trimmingCharacters(in: .whitespaces)
         if !search.isEmpty {
            let request = AppData.fetchedController.fetchRequest
            request.predicate = NSPredicate(format: "title CONTAINS[cd] %@", search)
            try? AppData.fetchedController.performFetch()
         }
      }
   }
   func searchBarCancelButtonClicked(_ searchBar: UISearchBar) {
      let request = AppData.fetchedController.fetchRequest
      request.predicate = nil
      try? AppData.fetchedController.performFetch()
   }
   @IBAction func editBooks(_ sender: UIBarButtonItem) {
      let editing = !tableView.isEditing
      tableView.setEditing(editing, animated: true)
   }
}

