
import UIKit
import CoreData

class EditBookViewController: UIViewController {
   @IBOutlet weak var bookTitle: UITextField!
   @IBOutlet weak var bookYear: UITextField!
   @IBOutlet weak var authorName: UILabel!
   var context: NSManagedObjectContext!
   var selectedAuthor: Authors!
   var selectedBook: Books!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      bookTitle.becomeFirstResponder()

      if selectedBook != nil {
         bookTitle.text = selectedBook.title
         bookYear.text = String(selectedBook.year)
         authorName.text = selectedBook.author?.name ?? "Undefined"

         selectedAuthor = selectedBook.author
      }
   }
   @IBAction func saveBook(_ sender: UIBarButtonItem) {
      let title = bookTitle.text!.trimmingCharacters(in: .whitespaces)
      let year = Int16(bookYear.text!)
      if title != "" && year != nil {
         Task(priority: .high) {
            await storeBook(title: title, year: year!)
         }
      }
   }
   func storeBook(title: String, year: Int16) async {
      await context.perform {
         if self.selectedBook != nil {
            self.selectedBook.title = title
            self.selectedBook.year = year
            self.selectedBook.author = self.selectedAuthor
         } else {
            let newBook = Books(context: self.context)
            newBook.title = title
            newBook.year = year
            newBook.author = self.selectedAuthor
         }
         do {
            try self.context.save()
         } catch {
            print("Error: \(error)")
         }
      }
      await MainActor.run {
         if let book = self.selectedBook {
            var currentSnapshot = AppData.dataSourceBooks.snapshot()
            currentSnapshot.reloadItems([book.objectID])
            AppData.dataSourceBooks.apply(currentSnapshot)
         }
         self.closeScene()
      }
   }
   func closeScene() {
      navigationController?.popViewController(animated: true)
   }
   @IBAction func backAuthor(_ segue: UIStoryboardSegue) {
      if segue.identifier == "backFromList" {
         let controller = segue.source as! AuthorsViewController
         selectedAuthor = controller.selectedAuthor
         authorName.text = selectedAuthor.name
      } else if segue.identifier == "backFromNew" {
         let controller = segue.source as! EditAuthorViewController
         selectedAuthor = controller.selectedAuthor
         authorName.text = selectedAuthor.name
      }
   }
}

