
import UIKit
import CoreData

class BooksViewController: UITableViewController, NSFetchedResultsControllerDelegate {
   var context: NSManagedObjectContext!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "booksCell")
      prepareDataSource()
      prepareFetchedController()
   }
   override func viewWillAppear(_ animated: Bool) {
      super.viewWillAppear(animated)
      do {
         try AppData.fetchedController.performFetch()
      } catch {
         print("Error")
      }
   }
   func prepareDataSource() {
      AppData.dataSourceBooks = MyDataSource(tableView: tableView) { tableView, indexPath, itemID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "booksCell", for: indexPath)
         if let item = try? self.context.existingObject(with: itemID) as? Books {
            var config = cell.defaultContentConfiguration()
            config.text = item.title

            var authors: String!
            if let list = item.author as? Set<Authors> {
               let listNames = list.map({ $0.name ?? "Undefined" })
               if !listNames.isEmpty {
                  authors = listNames.joined(separator: ", ")
               }
            }
            config.secondaryText = authors ?? "Undefined"
            config.secondaryTextProperties.color = .systemGray

            if let data = item.thumbnail, let image = UIImage(data:data){
               config.image = image
            } else {
               config.image = UIImage(named: "nothumbnail")
            }
            config.imageProperties.maximumSize = CGSize(width: 60, height: 60)
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func prepareFetchedController() {
      let request: NSFetchRequest<Books> = Books.fetchRequest()
      let sort = NSSortDescriptor(key: "title", ascending: true, selector: #selector(NSString.caseInsensitiveCompare(_:)))
      request.sortDescriptors = [sort]
      AppData.fetchedController = NSFetchedResultsController(fetchRequest: request, managedObjectContext: context, sectionNameKeyPath: nil, cacheName: nil)
      AppData.fetchedController.delegate = self
   }
   func controller(_ controller: NSFetchedResultsController<NSFetchRequestResult>, didChangeContentWith snapshot: NSDiffableDataSourceSnapshotReference) {
      let newsnapshot = snapshot as NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>
      AppData.dataSourceBooks.apply(newsnapshot, animatingDifferences: true)
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      performSegue(withIdentifier: "showEditBook", sender: self)
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "showEditBook" {
         if let path = tableView.indexPathForSelectedRow {
            if let itemID = AppData.dataSourceBooks.itemIdentifier(for: path) {
               if let item = try? context.existingObject(with: itemID) as? Books {
                  let controller = segue.destination as! EditBookViewController
                  controller.selectedBook = item
               }
            }
         }
      }
   }
   @IBAction func editBooks(_ sender: UIBarButtonItem) {
      let editing = !tableView.isEditing
      tableView.setEditing(editing, animated: true)
   }
}

