
import UIKit
import CoreData

class EditBookViewController: UIViewController {
   @IBOutlet weak var bookTitle: UITextField!
   @IBOutlet weak var bookYear: UITextField!
   @IBOutlet weak var authorName: UILabel!
   var context: NSManagedObjectContext!
   var selectedAuthors: [Authors] = []
   var selectedBook: Books!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      bookTitle.becomeFirstResponder()

      if selectedBook != nil {
         bookTitle.text = selectedBook.title
         bookYear.text = String(selectedBook.year)

         var authors: String!
         if let list = selectedBook.author as? Set<Authors> {
            let listNames = list.map({ $0.name ?? "Undefined" })
            if !listNames.isEmpty {
               authors = listNames.joined(separator: ", ")
            }
            selectedAuthors = Array(list)
         }
         authorName.text = authors ?? "Undefined"
      }
   }
   @IBAction func saveBook(_ sender: UIBarButtonItem) {
      let title = bookTitle.text!.trimmingCharacters(in: .whitespaces)
      let year = Int16(bookYear.text!)
      if title != "" && year != nil {
         Task(priority: .high) {
            await storeBook(title: title, year: year!)
         }
      }
   }
   func storeBook(title: String, year: Int16) async {
      await context.perform {
         if self.selectedBook != nil {
            self.selectedBook.title = title
            self.selectedBook.year = year
            self.selectedBook.author = NSSet(array: self.selectedAuthors)
         } else {
            let newBook = Books(context: self.context)
            newBook.title = title
            newBook.year = year
            newBook.author = NSSet(array: self.selectedAuthors)
         }
         do {
            try self.context.save()
         } catch {
            print("Error: \(error)")
         }
      }
      await MainActor.run {
         if let book = self.selectedBook {
            var currentSnapshot = AppData.dataSourceBooks.snapshot()
            currentSnapshot.reloadItems([book.objectID])
            AppData.dataSourceBooks.apply(currentSnapshot)
         }
         self.closeScene()
      }
   }
   override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
      if segue.identifier == "selectAuthor" {
         let controller = segue.destination as! AuthorsViewController
         controller.selectedAuthors = selectedAuthors
      }
   }
   func closeScene() {
      navigationController?.popViewController(animated: true)
   }
   @IBAction func backAuthor(_ segue: UIStoryboardSegue) {
      if segue.identifier == "backFromList" {
         let controller = segue.source as! AuthorsViewController
         selectedAuthors = controller.selectedAuthors
      } else if segue.identifier == "backFromNew" {
         let controller = segue.source as! EditAuthorViewController
         selectedAuthors.append(controller.selectedAuthor)
      }
      var authors: String!
      let listNames = selectedAuthors.map({ $0.name ?? "Undefined" })
      if !listNames.isEmpty {
         authors = listNames.joined(separator: ", ")
      }
      authorName.text = authors ?? "Undefined"
   }
}

