
import UIKit
import CoreData

class AuthorsViewController: UITableViewController {
   var context: NSManagedObjectContext!
   var selectedAuthor: Authors!

   override func viewDidLoad() {
      super.viewDidLoad()
      let app = UIApplication.shared
      let appDelegate = app.delegate as! AppDelegate
      context = appDelegate.persistentContainer.viewContext

      tableView.register(UITableViewCell.self, forCellReuseIdentifier: "authorsCell")
      prepareDataSource()
   }
   override func viewWillAppear(_ animated: Bool) {
      super.viewWillAppear(animated)
      Task(priority: .high) {
         await loadRequest()

         await MainActor.run {
            prepareSnapshot()
         }
      }
   }
   func prepareDataSource() {
      AppData.dataSourceAuthors = UITableViewDiffableDataSource<Sections, NSManagedObjectID>(tableView: tableView) { tableView, indexPath, itemID in
         let cell = tableView.dequeueReusableCell(withIdentifier: "authorsCell", for: indexPath)
         if let item = AppData.listOfAuthors.first(where: { $0.objectID == itemID }) {
            var config = cell.defaultContentConfiguration()
            config.text = item.name
            cell.contentConfiguration = config
         }
         return cell
      }
   }
   func loadRequest() async {
      await context.perform {
         let request: NSFetchRequest<Authors> = Authors.fetchRequest()
         do {
            AppData.listOfAuthors = try self.context.fetch(request)
         } catch {
            print("Error: \(error)")
         }
      }
   }
   func prepareSnapshot() {
      var snapshot = NSDiffableDataSourceSnapshot<Sections, NSManagedObjectID>()
      snapshot.appendSections([.main])
      snapshot.appendItems(AppData.listOfAuthors.map({ $0.objectID }))
      AppData.dataSourceAuthors.apply(snapshot, animatingDifferences: false)
   }
   override func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      if let itemID = AppData.dataSourceAuthors.itemIdentifier(for: indexPath) {
         if let item = AppData.listOfAuthors.first(where: { $0.objectID == itemID }) {
            selectedAuthor = item
         }
         performSegue(withIdentifier: "backFromList", sender: self)
      }
   }
}

